Attribute VB_Name = "Module1"
' This is a simple demo VB6 demo program to show how to use the wtCom.dll
' to communicate through the WT network. Its goal is to display in a small
' Edit Box Window the "SUMMARY" protocols sent periodicaly by the "Master"
' station on the Network.
'
' This program is part of the Win-Test logging software for HamRadio contesters:
' http://www.win-test.com
'
' This program is distributed in the hope that it will be useful, but
' WITHOUT ANY WARRANTY.
'
' The source code is licenced under the terms of the GNU Public General
' Licence (GPL): http://www.gnu.org/copyleft/gpl.html
'
' As I'm a total newcomer in Visual Basic, please don't blame me if
' the code looks unconventional to your eyes!
'
' (c) Mars 2006, Olivier Le Cam, f5mzn

Option Explicit

Public hOldWindowProc As Long                       ' The original wnd proc
Public Const WM_NETWORK_RECEIVE As Long = &H400     ' WM_USER message ID

' Win-test API
' WTCOM_init:    Initialize communication with the WT network
' WTCOM_close:   Close the communication with the WT network. Must be done before leaving the application.
' WTCOM_esc:     Escape characters which could mess up in a protocol. For example, the string:
'                   This is a "test"
'                will be converted into:
'                   This is a \"Test\"
'                It's a good idea to use this function for each string wich will be sent to the WT network
' WTCOM_parsevb: Parse a protocol and extract each fields. Convert espaced characters if any.
'                See the example below.
' WTCOM_send:    Send a protocol string to the WT network.
Public Declare Sub WTCOM_init Lib "wtCom.dll" Alias "_WTCOM_init@16" (ByVal hwnd As Long, ByVal nMessage As Long, ByVal lpszBroadcastAddress As String, ByVal nPort As Long)
Public Declare Sub WTCOM_send Lib "wtCom.dll" Alias "_WTCOM_send@4" (ByVal str As String)
Public Declare Function WTCOM_esc Lib "wtCom.dll" Alias "_WTCOM_esc@8" (ByVal source As String, ByVal dest As String) As String
Public Declare Function WTCOM_parse Lib "wtCom.dll" Alias "_WTCOM_parse@8" (argv As Long, ByVal szCmd As String) As Long
Public Declare Function WTCOM_parsevb Lib "wtCom.dll" Alias "_WTCOM_parsevb@8" (args() As String, ByVal szCmd As String) As Long
Public Declare Sub WTCOM_close Lib "\\Calumet\Release\wtCom.dll" Alias "_WTCOM_close@0" ()

' Function to retrieve the address of the current Message-Handling routine
Declare Function GetWindowLong Lib "user32" Alias "GetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long) As Long
' Function to define the address of the Message-Handling routine
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long
' Function to execute a function residing at a specific memory address
Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
' Convert a valid pointer into a string
Private Declare Function lstrcpyA Lib "kernel32" (ByVal RetVal As String, ByVal Ptr As Long) As Long
Private Declare Function lstrlenA Lib "kernel32" (ByVal Ptr As Any) As Long
Public Function GetStrFromPtrA(ByVal lpszA As Long) As String
   GetStrFromPtrA = String$(lstrlenA(ByVal lpszA), 0)
   Call lstrcpyA(ByVal GetStrFromPtrA, ByVal lpszA)
End Function

' Subclassed WindowProc function to intercept the Windows Messages sent by the DLL when a new protocol
' is sent through the WT network
Public Function MyWndProc(ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    ' Watch for the messages to come in from the WTCOM DLL
    If Msg = WM_NETWORK_RECEIVE Then
        ' Convert lParam which is pointer to the protocol into a string
        Dim protocol As String
        protocol = GetStrFromPtrA(lParam)
        
        ' As "protocol" will be destructed later by the WTCOM_parsevb call, we prepare a backup copy
        ' in a different variable
        Dim protocol2 As String
        protocol2 = protocol
        
        ' Extract each field of the protocol
        Dim fields(0 To 32) As String, i As Long
        Dim nb As Long
        
        For i = 0 To 32 ' up to 32 fields are supported by the DLL
            ' Alloc memory for each fields
            fields(i) = Space(1024)
        Next
        nb = WTCOM_parsevb(fields(), protocol)  ' NB: this call is destructive for the "protocol" variable
        For i = 0 To (nb - 1)
            ' Remove the terminating \0 character (VC++ DLL)
            fields(i) = Mid$(fields(i), 1, InStr(1, fields(i), Chr$(0)) - 1)
        Next

        ' Filter to display only the current summary score sent by the 'master' WT station
        If fields(0) = "SUMMARY:" Then
            Form1.EditScore = protocol2 + Chr$(13) + Chr$(10) + Form1.EditScore
        End If
        
        ' Ending function
        MyWndProc = 1
        Exit Function
    End If
    
    ' Here, we forward all irrelevant messages on to the default message handler.
    MyWndProc = CallWindowProc(hOldWindowProc, hwnd, Msg, wParam, lParam)
End Function
